-- StartNoTest
DELIMITER //

DROP TRIGGER IF EXISTS SampleChange//
CREATE TRIGGER SampleChange BEFORE UPDATE ON Sample
FOR EACH ROW
  BEGIN
  DECLARE log_message varchar(500) CHARACTER SET utf8;
  SET log_message = CONCAT_WS(', ',
    CASE WHEN (NEW.accession IS NULL) <> (OLD.accession IS NULL) OR NEW.accession <> OLD.accession THEN CONCAT('accession: ', COALESCE(OLD.accession, 'n/a'), ' → ', COALESCE(NEW.accession, 'n/a')) END,
    CASE WHEN (NEW.alias IS NULL) <> (OLD.alias IS NULL) OR NEW.alias <> OLD.alias AND (OLD.alias NOT LIKE 'TEMPORARY%') THEN CONCAT('alias: ', COALESCE(OLD.alias, 'n/a'), ' → ', COALESCE(NEW.alias, 'n/a')) END,
    CASE WHEN (NEW.description IS NULL) <> (OLD.description IS NULL) OR NEW.description <> OLD.description THEN CONCAT('description: ', OLD.description, ' → ', NEW.description) END,
    CASE WHEN (NEW.identificationBarcode IS NULL) <> (OLD.identificationBarcode IS NULL) OR NEW.identificationBarcode <> OLD.identificationBarcode THEN CONCAT('barcode: ', COALESCE(OLD.identificationBarcode, 'n/a'), ' → ', COALESCE(NEW.identificationBarcode, 'n/a')) END,
    CASE WHEN (NEW.locationBarcode IS NULL) <> (OLD.locationBarcode IS NULL) OR NEW.locationBarcode <> OLD.locationBarcode THEN CONCAT('location: ', COALESCE(OLD.locationBarcode, 'n/a'), ' → ', COALESCE(NEW.locationBarcode, 'n/a')) END,
    CASE WHEN NEW.project_projectId <> OLD.project_projectId THEN CONCAT('project: ', (SELECT name FROM Project WHERE projectId = OLD.project_projectId), ' → ', (SELECT name FROM Project WHERE projectId = NEW.project_projectId)) END,
    CASE WHEN (NEW.qcPassed IS NULL) <> (OLD.qcPassed IS NULL) OR NEW.qcPassed <> OLD.qcPassed THEN CONCAT('qcPassed: ', COALESCE(OLD.qcPassed, 'n/a'), ' → ', COALESCE(NEW.qcPassed, 'n/a')) END,
    CASE WHEN NEW.sampleType <> OLD.sampleType THEN CONCAT('type: ', OLD.sampleType, ' → ', NEW.sampleType) END,
    CASE WHEN NEW.scientificName <> OLD.scientificName THEN CONCAT('scientific name: ', OLD.scientificName, ' → ', NEW.scientificName) END,
    CASE WHEN (NEW.taxonIdentifier IS NULL) <> (OLD.taxonIdentifier IS NULL) OR NEW.taxonIdentifier <> OLD.taxonIdentifier THEN CONCAT('taxon: ', COALESCE(OLD.taxonIdentifier, 'n/a'), ' → ', COALESCE(NEW.taxonIdentifier, 'n/a')) END,
    CASE WHEN NEW.discarded <> OLD.discarded THEN CONCAT('discarded: ', IF(OLD.discarded = 0, 'No', 'Yes'), ' → ', IF(NEW.discarded = 0, 'No', 'Yes')) END,
    CASE WHEN (NEW.concentration IS NULL) <> (OLD.concentration IS NULL) OR NEW.concentration <> OLD.concentration THEN CONCAT('concentration: ', COALESCE(decimalToString(OLD.concentration), 'n/a'), ' → ', COALESCE(decimalToString(NEW.concentration), 'n/a')) END, 
    CASE WHEN (NEW.initialVolume IS NULL) <> (OLD.initialVolume IS NULL) OR NEW.initialVolume <> OLD.initialVolume THEN CONCAT('initial volume: ', COALESCE(decimalToString(OLD.initialVolume), 'n/a'), ' → ', COALESCE(decimalToString(NEW.initialVolume), 'n/a')) END,
    CASE WHEN (NEW.volume IS NULL) <> (OLD.volume IS NULL) OR NEW.volume <> OLD.volume THEN CONCAT('volume: ', COALESCE(decimalToString(OLD.volume), 'n/a'), ' → ', COALESCE(decimalToString(NEW.volume), 'n/a')) END,
    CASE WHEN (NEW.concentrationUnits IS NULL) <> (OLD.concentrationUnits IS NULL) OR NEW.concentrationUnits <> OLD.concentrationUnits THEN CONCAT('concentration units: ', COALESCE(OLD.concentrationUnits, 'n/a'), ' → ', COALESCE(NEW.concentrationUnits, 'n/a')) END,
    CASE WHEN (NEW.volumeUnits IS NULL) <> (OLD.volumeUnits IS NULL) OR NEW.volumeUnits <> OLD.volumeUnits THEN CONCAT('volume units: ', COALESCE(OLD.volumeUnits, 'n/a'), ' → ', COALESCE(NEW.volumeUnits, 'n/a')) END,
    CASE WHEN (NEW.requisitionId IS NULL) <> (OLD.requisitionId IS NULL) OR NEW.requisitionId <> OLD.requisitionId THEN CONCAT('requisition ID: ', COALESCE(OLD.requisitionId, 'n/a'), ' → ', COALESCE(NEW.requisitionId, 'n/a')) END,
    CASE WHEN (NEW.sequencingControlTypeId IS NULL) <> (OLD.sequencingControlTypeId IS NULL) OR NEW.sequencingControlTypeId <> OLD.sequencingControlTypeId THEN CONCAT('sequencing control type: ', COALESCE((SELECT alias FROM SequencingControlType WHERE sequencingControlTypeId = OLD.sequencingControlTypeId), 'n/a'), ' → ', COALESCE((SELECT alias FROM SequencingControlType WHERE sequencingControlTypeId = NEW.sequencingControlTypeId), 'n/a')) END);
  IF log_message IS NOT NULL AND log_message <> '' THEN
    INSERT INTO SampleChangeLog(sampleId, columnsChanged, userId, message, changeTime) VALUES (
      NEW.sampleId,
      COALESCE(CONCAT_WS(',',
        CASE WHEN (NEW.accession IS NULL) <> (OLD.accession IS NULL) OR NEW.accession <> OLD.accession THEN 'accession' END,
        CASE WHEN (NEW.alias IS NULL) <> (OLD.alias IS NULL) OR NEW.alias <> OLD.alias AND (OLD.alias NOT LIKE 'TEMPORARY%') THEN 'alias' END,
        CASE WHEN (NEW.description IS NULL) <> (OLD.description IS NULL) OR NEW.description <> OLD.description THEN 'description' END,
        CASE WHEN (NEW.identificationBarcode IS NULL) <> (OLD.identificationBarcode IS NULL) OR NEW.identificationBarcode <> OLD.identificationBarcode THEN 'barcode' END,
        CASE WHEN NEW.locationBarcode <> OLD.locationBarcode THEN 'locationBarcode' END,
        CASE WHEN NEW.project_projectId <> OLD.project_projectId THEN 'project_projectId' END,
        CASE WHEN (NEW.qcPassed IS NULL) <> (OLD.qcPassed IS NULL) OR NEW.qcPassed <> OLD.qcPassed THEN 'qcPassed' END,
        CASE WHEN NEW.sampleType <> OLD.sampleType THEN 'sampleType' END,
        CASE WHEN NEW.scientificName <> OLD.scientificName THEN 'scientificName' END,
        CASE WHEN (NEW.taxonIdentifier IS NULL) <> (OLD.taxonIdentifier IS NULL) OR NEW.taxonIdentifier <> OLD.taxonIdentifier THEN 'taxonIdentifier' END,
        CASE WHEN NEW.discarded <> OLD.discarded THEN 'discarded' END,
        CASE WHEN (NEW.concentration IS NULL) <> (OLD.concentration IS NULL) OR NEW.concentration <> OLD.concentration THEN 'concentration' END,
        CASE WHEN (NEW.initialVolume IS NULL) <> (OLD.initialVolume IS NULL) OR NEW.initialVolume <> OLD.initialVolume THEN 'initial volume' END,
        CASE WHEN (NEW.volume IS NULL) <> (OLD.volume IS NULL) OR NEW.volume <> OLD.volume THEN 'volume' END,
        CASE WHEN (NEW.concentrationUnits IS NULL) <> (OLD.concentrationUnits IS NULL) OR NEW.concentrationUnits <> OLD.concentrationUnits THEN CONCAT(NEW.name, ' concentrationUnits') END,
        CASE WHEN (NEW.volumeUnits IS NULL) <> (OLD.volumeUnits IS NULL) OR NEW.volumeUnits <> OLD.volumeUnits THEN CONCAT(NEW.name, ' volumeUnits') END,
        CASE WHEN (NEW.requisitionId IS NULL) <> (OLD.requisitionId IS NULL) OR NEW.requisitionId <> OLD.requisitionId THEN 'requisitionId' END,
        CASE WHEN (NEW.sequencingControlTypeId IS NULL) <> (OLD.sequencingControlTypeId IS NULL) OR NEW.sequencingControlTypeId <> OLD.sequencingControlTypeId THEN 'sequencingControlTypeId' END
  ), ''),
      NEW.lastModifier,
      log_message,
      NEW.lastModified
      );
  END IF;
  END//

DROP TRIGGER IF EXISTS DetailedSampleChange//
CREATE TRIGGER DetailedSampleChange BEFORE UPDATE ON DetailedSample
FOR EACH ROW
  BEGIN
  DECLARE log_message varchar(500) CHARACTER SET utf8;
  SET log_message = CONCAT_WS(', ',
     CASE WHEN NEW.archived <> OLD.archived THEN CONCAT('archived: ', OLD.archived, ' → ', NEW.archived) END,
     CASE WHEN (NEW.groupDescription IS NULL) <> (OLD.groupDescription IS NULL) OR NEW.groupDescription <> OLD.groupDescription THEN CONCAT('group description: ', COALESCE(OLD.groupDescription, 'n/a'), ' → ', COALESCE(NEW.groupDescription, 'n/a')) END,
     CASE WHEN (NEW.groupId IS NULL) <> (OLD.groupId IS NULL) OR NEW.groupId <> OLD.groupId THEN CONCAT('group id: ', COALESCE(OLD.groupId, 'n/a'), ' → ', COALESCE(NEW.groupId, 'n/a')) END,
     CASE WHEN (NEW.parentId IS NULL) <> (OLD.parentId IS NULL) OR NEW.parentId <> OLD.parentId THEN CONCAT('parent: ', (SELECT name FROM Sample WHERE sampleId = OLD.parentId), ' → ', (SELECT name FROM Sample WHERE sampleId = NEW.parentId)) END,
     CASE WHEN (NEW.detailedQcStatusId IS NULL) <> (OLD.detailedQcStatusId IS NULL) OR NEW.detailedQcStatusId <> OLD.detailedQcStatusId THEN CONCAT('QC Status: ', COALESCE((SELECT description FROM DetailedQcStatus WHERE detailedQcStatusId = OLD.detailedQcStatusId), 'n/a'), ' → ', COALESCE((SELECT description FROM DetailedQcStatus WHERE detailedQcStatusId = NEW.detailedQcStatusId), 'n/a')) END,
     CASE WHEN (NEW.detailedQcStatusNote IS NULL) <> (OLD.detailedQcStatusNote IS NULL) OR NEW.detailedQcStatusNote <> OLD.detailedQcStatusNote THEN CONCAT('QC Status Note: ', COALESCE(OLD.detailedQcStatusNote, 'n/a'), ' → ', COALESCE(NEW.detailedQcStatusNote, 'n/a')) END,
     CASE WHEN NEW.sampleClassId <> OLD.sampleClassId THEN CONCAT('class: ', (SELECT alias FROM SampleClass WHERE sampleClassId = OLD.sampleClassId), ' → ', (SELECT alias FROM SampleClass WHERE sampleClassId = NEW.sampleClassId)) END,
     CASE WHEN (NEW.subprojectId IS NULL) <> (OLD.subprojectId IS NULL) OR NEW.subprojectId <> OLD.subprojectId THEN CONCAT('subproject: ', COALESCE((SELECT alias FROM Subproject WHERE subprojectId = OLD.subprojectId), 'n/a'), ' → ', COALESCE((SELECT alias FROM Subproject WHERE subprojectId = NEW.subprojectId), 'n/a')) END,
     CASE WHEN (NEW.volumeUsed IS NULL) <> (OLD.volumeUsed IS NULL) OR NEW.volumeUsed <> OLD.volumeUsed THEN CONCAT('volume used: ', COALESCE(decimalToString(OLD.volumeUsed), 'n/a'), ' → ', COALESCE(decimalToString(NEW.volumeUsed), 'n/a')) END,
     CASE WHEN (NEW.ngUsed IS NULL) <> (OLD.ngUsed IS NULL) OR NEW.ngUsed <> OLD.ngUsed THEN CONCAT('ng used: ', COALESCE(decimalToString(OLD.ngUsed), 'n/a'), ' → ', COALESCE(decimalToString(NEW.ngUsed), 'n/a')) END,
     CASE WHEN (NEW.creationDate IS NULL) <> (OLD.creationDate IS NULL) OR NEW.creationDate <> OLD.creationDate THEN CONCAT('creationDate: ', COALESCE(OLD.creationDate, 'n/a'), ' → ', COALESCE(NEW.creationDate, 'n/a')) END);
  IF log_message IS NOT NULL AND log_message <> '' THEN
    INSERT INTO SampleChangeLog(sampleId, columnsChanged, userId, message, changeTime) 
    SELECT
      NEW.sampleId,
      COALESCE(CONCAT_WS(',',
        CASE WHEN NEW.archived <> OLD.archived THEN 'archived' END,
        CASE WHEN (NEW.groupDescription IS NULL) <> (OLD.groupDescription IS NULL) OR NEW.groupDescription <> OLD.groupDescription THEN 'groupDescription' END,
        CASE WHEN (NEW.groupId IS NULL) <> (OLD.groupId IS NULL) OR NEW.groupId <> OLD.groupId THEN 'groupId' END,
        CASE WHEN (NEW.parentId IS NULL) <> (OLD.parentId IS NULL) OR NEW.parentId <> OLD.parentId THEN 'parentId' END,
        CASE WHEN (NEW.detailedQcStatusId IS NULL) <> (OLD.detailedQcStatusId IS NULL) OR NEW.detailedQcStatusId <> OLD.detailedQcStatusId THEN 'detailedQcStatusId' END,
        CASE WHEN (NEW.detailedQcStatusNote IS NULL) <> (OLD.detailedQcStatusNote IS NULL) OR NEW.detailedQcStatusNote <> OLD.detailedQcStatusNote THEN 'detailedQcStatusNote' END,
        CASE WHEN NEW.sampleClassId <> OLD.sampleClassId THEN 'sampleClassId' END,
        CASE WHEN (NEW.subprojectId IS NULL) <> (OLD.subprojectId IS NULL) OR NEW.subprojectId <> OLD.subprojectId THEN 'subprojectId' END,
        CASE WHEN (NEW.volumeUsed IS NULL) <> (OLD.volumeUsed IS NULL) OR NEW.volumeUsed <> OLD.volumeUsed THEN 'volume used' END,
        CASE WHEN (NEW.ngUsed IS NULL) <> (OLD.ngUsed IS NULL) OR NEW.ngUsed <> OLD.ngUsed THEN 'ng used' END,
        CASE WHEN (NEW.creationDate IS NULL) <> (OLD.creationDate IS NULL) OR NEW.creationDate <> OLD.creationDate THEN 'creationDate' END
      ), ''),
      lastModifier,
      log_message,
      lastModified
    FROM Sample WHERE sampleId = NEW.sampleId;
  END IF;
  IF (NEW.parentId IS NULL) <> (OLD.parentId IS NULL) OR NEW.parentId <> OLD.parentId THEN
    CALL updateSampleHierarchy(NEW.sampleId);
  END IF;
  END//

DROP TRIGGER IF EXISTS SampleAliquotChange//
CREATE TRIGGER SampleAliquotChange BEFORE UPDATE ON SampleAliquot
FOR EACH ROW
  BEGIN
  DECLARE log_message varchar(500) CHARACTER SET utf8;
  SET log_message = CONCAT_WS(', ',
     CASE WHEN (NEW.samplePurposeId IS NULL) <> (OLD.samplePurposeId IS NULL) OR NEW.samplePurposeId <> OLD.samplePurposeId THEN CONCAT('purpose: ', COALESCE((SELECT alias FROM SamplePurpose WHERE samplePurposeId = OLD.samplePurposeId), 'n/a'), ' → ', COALESCE((SELECT alias FROM SamplePurpose WHERE samplePurposeId = NEW.samplePurposeId), 'n/a')) END);
  IF log_message IS NOT NULL AND log_message <> '' THEN
    INSERT INTO SampleChangeLog(sampleId, columnsChanged, userId, message, changeTime)
    SELECT
      NEW.sampleId,
      COALESCE(CONCAT_WS(',',
        CASE WHEN (NEW.samplePurposeId IS NULL) <> (OLD.samplePurposeId IS NULL) OR NEW.samplePurposeId <> OLD.samplePurposeId THEN 'samplePurposeId' END
      ), ''),
      lastModifier,
      log_message,
      lastModified
    FROM Sample WHERE sampleId = NEW.sampleId;
  END IF;
  END//

DROP TRIGGER IF EXISTS SampleSlideChange//
CREATE TRIGGER SampleSlideChange BEFORE UPDATE ON SampleSlide
FOR EACH ROW
  BEGIN
  DECLARE log_message varchar(500) CHARACTER SET utf8;
  SET log_message = CONCAT_WS(', ',
     CASE WHEN NEW.slides <> OLD.slides THEN CONCAT('slides: ', OLD.slides, ' → ', NEW.slides) END,
     CASE WHEN (NEW.discards IS NULL) <> (OLD.discards IS NULL) OR NEW.discards <> OLD.discards THEN CONCAT('discards: ', COALESCE(OLD.discards, 'n/a'), ' → ', COALESCE(NEW.discards, 'n/a')) END,
     CASE WHEN (NEW.thickness IS NULL) <> (OLD.thickness IS NULL) OR NEW.thickness <> OLD.thickness THEN CONCAT('thickness: ', COALESCE(OLD.thickness, 'n/a'), ' → ', COALESCE(NEW.thickness, 'n/a')) END,
     CASE WHEN (NEW.stain IS NULL) <> (OLD.stain IS NULL) OR NEW.stain <> OLD.stain THEN CONCAT('stain: ', COALESCE((SELECT name FROM Stain WHERE stainId = OLD.stain), 'none'), ' → ', COALESCE((SELECT name FROM Stain WHERE stainId = NEW.stain), 'none')) END,
     CASE WHEN (NEW.percentTumour IS NULL) <> (OLD.percentTumour IS NULL) OR NEW.percentTumour <> OLD.percentTumour THEN CONCAT('percent tumour: ', COALESCE(OLD.percentTumour, 'n/a'), ' → ', COALESCE(NEW.percentTumour, 'n/a')) END,
     CASE WHEN (NEW.percentNecrosis IS NULL) <> (OLD.percentNecrosis IS NULL) OR NEW.percentNecrosis <> OLD.percentNecrosis THEN CONCAT('percent necrosis: ', COALESCE(OLD.percentNecrosis, 'n/a'), ' → ', COALESCE(NEW.percentNecrosis, 'n/a')) END,
     CASE WHEN (NEW.markedAreaSize IS NULL) <> (OLD.markedAreaSize IS NULL) OR NEW.markedAreaSize <> OLD.markedAreaSize THEN CONCAT('marked area size: ', COALESCE(OLD.markedAreaSize, 'n/a'), ' → ', COALESCE(NEW.markedAreaSize, 'n/a')) END,
     CASE WHEN (NEW.markedAreaPercentTumour IS NULL) <> (OLD.markedAreaPercentTumour IS NULL) OR NEW.markedAreaPercentTumour <> OLD.markedAreaPercentTumour THEN CONCAT('marked area percent tumour: ', COALESCE(OLD.markedAreaPercentTumour, 'n/a'), ' → ', COALESCE(NEW.markedAreaPercentTumour, 'n/a')) END);
  IF log_message IS NOT NULL AND log_message <> '' THEN
    INSERT INTO SampleChangeLog(sampleId, columnsChanged, userId, message, changeTime)
    SELECT
      NEW.sampleId,
      COALESCE(CONCAT_WS(',',
        CASE WHEN NEW.slides <> OLD.slides THEN 'slides' END,
        CASE WHEN (NEW.discards IS NULL) <> (OLD.discards IS NULL) OR NEW.discards <> OLD.discards THEN 'discards' END,
        CASE WHEN (NEW.thickness IS NULL) <> (OLD.thickness IS NULL) OR NEW.thickness <> OLD.thickness THEN 'thickness' END,
        CASE WHEN (NEW.stain IS NULL) <> (OLD.stain IS NULL) OR NEW.stain <> OLD.stain THEN 'stain' END,
        CASE WHEN (NEW.percentTumour IS NULL) <> (OLD.percentTumour IS NULL) OR NEW.percentTumour <> OLD.percentTumour THEN 'percentTumour' END,
        CASE WHEN (NEW.percentNecrosis IS NULL) <> (OLD.percentNecrosis IS NULL) OR NEW.percentNecrosis <> OLD.percentNecrosis THEN 'percentNecrosis' END,
        CASE WHEN (NEW.markedAreaSize IS NULL) <> (OLD.markedAreaSize IS NULL) OR NEW.markedAreaSize <> OLD.markedAreaSize THEN 'markedAreaSize' END,
        CASE WHEN (NEW.markedAreaPercentTumour IS NULL) <> (OLD.markedAreaPercentTumour IS NULL) OR NEW.markedAreaPercentTumour <> OLD.markedAreaPercentTumour THEN 'markedAreaPercentTumour' END
      ), ''),
      lastModifier,
      log_message,
      lastModified
    FROM Sample WHERE sampleId = NEW.sampleId;
  END IF;
  END//

DROP TRIGGER IF EXISTS SampleTissuePieceChange//
CREATE TRIGGER SampleTissuePieceChange BEFORE UPDATE ON SampleTissuePiece
FOR EACH ROW
  BEGIN
  DECLARE log_message varchar(500) CHARACTER SET utf8;
  SET log_message = CONCAT_WS(', ',
     CASE WHEN NEW.slidesConsumed <> OLD.slidesConsumed THEN CONCAT('slides: ', OLD.slidesConsumed, ' → ', NEW.slidesConsumed) END,
     CASE WHEN NEW.tissuePieceType <> OLD.tissuePieceType THEN CONCAT('type: ', (SELECT name FROM TissuePieceType WHERE tissuePieceTypeId = OLD.tissuePieceType), ' → ', (SELECT name FROM TissuePieceType WHERE tissuePieceTypeId = NEW.tissuePieceType)) END,
     CASE WHEN (NEW.referenceSlideId IS NULL) <> (OLD.referenceSlideId IS NULL) OR NEW.referenceSlideId <> OLD.referenceSlideId THEN CONCAT('reference slide: ', COALESCE((SELECT name FROM Sample WHERE sampleId = OLD.referenceSlideId), 'n/a'), ' → ', COALESCE((SELECT name FROM Sample WHERE sampleId = NEW.referenceSlideId), 'n/a')) END);
  IF log_message IS NOT NULL AND log_message <> '' THEN
    INSERT INTO SampleChangeLog(sampleId, columnsChanged, userId, message, changeTime)
    SELECT
      NEW.sampleId,
      COALESCE(CONCAT_WS(',',
        CASE WHEN NEW.slidesConsumed <> OLD.slidesConsumed THEN 'slides' END,
        CASE WHEN NEW.tissuePieceType <> OLD.tissuePieceType THEN 'tissuePieceType' END,
        CASE WHEN (NEW.referenceSlideId IS NULL) <> (OLD.referenceSlideId IS NULL) OR NEW.referenceSlideId <> OLD.referenceSlideId THEN 'referenceSlideId' END
      ), ''),
      lastModifier,
      log_message,
      lastModified
    FROM Sample WHERE sampleId = NEW.sampleId;
  END IF;
  END//

DROP TRIGGER IF EXISTS SampleStockChange//
CREATE TRIGGER SampleStockChange BEFORE UPDATE ON SampleStock
FOR EACH ROW
  BEGIN
  DECLARE log_message varchar(500) CHARACTER SET utf8;
  SET log_message = CONCAT_WS(', ',
    CASE WHEN NEW.strStatus <> OLD.strStatus THEN CONCAT('STR status: ', OLD.strStatus, ' → ', NEW.strStatus) END,
    CASE WHEN (NEW.referenceSlideId IS NULL) <> (OLD.referenceSlideId IS NULL) OR NEW.referenceSlideId <> OLD.referenceSlideId THEN CONCAT('reference slide: ', COALESCE((SELECT name FROM Sample WHERE sampleId = OLD.referenceSlideId), 'n/a'), ' → ', COALESCE((SELECT name FROM Sample WHERE sampleId = NEW.referenceSlideId), 'n/a')) END);
  IF log_message IS NOT NULL AND log_message <> '' THEN
    INSERT INTO SampleChangeLog(sampleId, columnsChanged, userId, message, changeTime)
    SELECT
      NEW.sampleId,
      COALESCE(CONCAT_WS(',',
         CASE WHEN NEW.strStatus <> OLD.strStatus THEN 'strStatus' END,
         CASE WHEN (NEW.referenceSlideId IS NULL) <> (OLD.referenceSlideId IS NULL) OR NEW.referenceSlideId <> OLD.referenceSlideId THEN 'referenceSlideId' END
      ), ''),
      lastModifier,
      log_message,
      lastModified
    FROM Sample WHERE sampleId = NEW.sampleId;
  END IF;
  END//

DROP TRIGGER IF EXISTS SampleTissueChange//
CREATE TRIGGER SampleTissueChange BEFORE UPDATE ON SampleTissue
FOR EACH ROW
  BEGIN
  DECLARE log_message varchar(500) CHARACTER SET utf8;
  SET log_message = CONCAT_WS(', ',
    CASE WHEN (NEW.secondaryIdentifier IS NULL) <> (OLD.secondaryIdentifier IS NULL) OR NEW.secondaryIdentifier <> OLD.secondaryIdentifier THEN CONCAT('secondary identifier: ', COALESCE(OLD.secondaryIdentifier, 'n/a'), ' → ', COALESCE(NEW.secondaryIdentifier, 'n/a')) END,
    CASE WHEN (NEW.labId IS NULL) <> (OLD.labId IS NULL) OR NEW.labId <> OLD.labId THEN CONCAT('lab: ', COALESCE((SELECT alias FROM Lab WHERE labId = OLD.labId), 'n/a'), ' → ', COALESCE((SELECT alias FROM Lab WHERE labId = NEW.labId), 'n/a')) END,
    CASE WHEN (NEW.passageNumber IS NULL) <> (OLD.passageNumber IS NULL) OR NEW.passageNumber <> OLD.passageNumber OR (NEW.timesReceived IS NULL) <> (OLD.timesReceived IS NULL) OR NEW.timesReceived <> OLD.timesReceived OR (NEW.tubeNumber IS NULL) <> (OLD.tubeNumber IS NULL) OR NEW.tubeNumber <> OLD.tubeNumber THEN CONCAT('passage: ', COALESCE(OLD.passageNumber, 'n/a'), '-', COALESCE(OLD.timesReceived, 'n/a'), '-', COALESCE(OLD.tubeNumber, 'n/a'), ' → ', COALESCE(NEW.passageNumber, 'n/a'), '-', COALESCE(NEW.timesReceived, 'n/a'), '-', COALESCE(NEW.tubeNumber, 'n/a')) END,
    CASE WHEN (NEW.region IS NULL) <> (OLD.region IS NULL) OR NEW.region <> OLD.region THEN CONCAT('region: ', COALESCE(OLD.region, 'n/a'), ' → ', COALESCE(NEW.region, 'n/a')) END,
    CASE WHEN (NEW.tissueMaterialId IS NULL) <> (OLD.tissueMaterialId IS NULL) OR NEW.tissueMaterialId <> OLD.tissueMaterialId THEN CONCAT('material: ', COALESCE((SELECT alias FROM TissueMaterial WHERE tissueMaterialId = OLD.tissueMaterialId), 'n/a'), ' → ', COALESCE((SELECT alias FROM TissueMaterial WHERE tissueMaterialId = NEW.tissueMaterialId), 'n/a')) END,
    CASE WHEN (NEW.tissueOriginId IS NULL) <> (OLD.tissueOriginId IS NULL) OR NEW.tissueOriginId <> OLD.tissueOriginId THEN CONCAT('origin: ', COALESCE((SELECT alias FROM TissueOrigin WHERE tissueOriginId = OLD.tissueOriginId), 'n/a'), ' → ', COALESCE((SELECT alias FROM TissueOrigin WHERE tissueOriginId = NEW.tissueOriginId), 'n/a')) END,
    CASE WHEN (NEW.tissueTypeId IS NULL) <> (OLD.tissueTypeId IS NULL) OR NEW.tissueTypeId <> OLD.tissueTypeId THEN CONCAT('type: ', COALESCE((SELECT alias FROM TissueType WHERE tissueTypeId = OLD.tissueTypeId), 'n/a'), ' → ', COALESCE((SELECT alias FROM TissueType WHERE tissueTypeId = NEW.tissueTypeId), 'n/a')) END);
  IF log_message IS NOT NULL AND log_message <> '' THEN
    INSERT INTO SampleChangeLog(sampleId, columnsChanged, userId, message, changeTime)
    SELECT
      NEW.sampleId,
      COALESCE(CONCAT_WS(',',
         CASE WHEN (NEW.secondaryIdentifier IS NULL) <> (OLD.secondaryIdentifier IS NULL) OR NEW.secondaryIdentifier <> OLD.secondaryIdentifier THEN 'secondaryIdentifier' END,
         CASE WHEN (NEW.labId IS NULL) <> (OLD.labId IS NULL) OR NEW.labId <> OLD.labId THEN 'labId' END,
         CASE WHEN (NEW.passageNumber IS NULL) <> (OLD.passageNumber IS NULL) OR NEW.passageNumber <> OLD.passageNumber THEN 'passageNumber' END,
         CASE WHEN (NEW.region IS NULL) <> (OLD.region IS NULL) OR NEW.region <> OLD.region THEN 'region' END,
         CASE WHEN (NEW.timesReceived IS NULL) <> (OLD.timesReceived IS NULL) OR NEW.timesReceived <> OLD.timesReceived THEN 'timesReceived' END,
         CASE WHEN (NEW.tissueMaterialId IS NULL) <> (OLD.tissueMaterialId IS NULL) OR NEW.tissueMaterialId <> OLD.tissueMaterialId THEN 'tissueMaterialId' END,
         CASE WHEN (NEW.tissueOriginId IS NULL) <> (OLD.tissueOriginId IS NULL) OR NEW.tissueOriginId <> OLD.tissueOriginId THEN 'tissueOriginId' END,
         CASE WHEN (NEW.tissueTypeId IS NULL) <> (OLD.tissueTypeId IS NULL) OR NEW.tissueTypeId <> OLD.tissueTypeId THEN 'tissueTypeId' END,
         CASE WHEN (NEW.tubeNumber IS NULL) <> (OLD.tubeNumber IS NULL) OR NEW.tubeNumber <> OLD.tubeNumber THEN 'tubeNumber' END
      ), ''),
      lastModifier,
      log_message,
      lastModified
    FROM Sample WHERE sampleId = NEW.sampleId;
  END IF;
  END//

DROP TRIGGER IF EXISTS IdentityChange//
CREATE TRIGGER IdentityChange BEFORE UPDATE ON Identity
FOR EACH ROW
  BEGIN
  DECLARE log_message varchar(500) CHARACTER SET utf8;
  SET log_message = CONCAT_WS(', ',
    CASE WHEN NEW.donorSex <> OLD.donorSex THEN CONCAT('donor sex: ', OLD.donorSex, ' → ', NEW.donorSex) END,
    CASE WHEN NEW.consentLevel <> OLD.consentLevel THEN CONCAT('consent level: ', OLD.consentLevel, ' → ', NEW.consentLevel) END,
    CASE WHEN NEW.externalName <> OLD.externalName THEN CONCAT('externalName: ', OLD.externalName, ' → ', NEW.externalName) END);
  IF log_message IS NOT NULL AND log_message <> '' THEN
    INSERT INTO SampleChangeLog(sampleId, columnsChanged, userId, message, changeTime)
    SELECT
      NEW.sampleId,
      COALESCE(CONCAT_WS(',',
        CASE WHEN NEW.donorSex <> OLD.donorSex THEN 'donorSex' END,
        CASE WHEN NEW.consentLevel <> OLD.consentLevel THEN 'consentLevel' END,
        CASE WHEN NEW.externalName <> OLD.externalName THEN 'externalName' END
      ), ''),
      lastModifier,
      log_message,
      lastModified
    FROM Sample WHERE sampleId = NEW.sampleId;
  END IF;
  END//

DROP TRIGGER IF EXISTS SampleSingleCellChange//
CREATE TRIGGER SampleSingleCellChange BEFORE UPDATE ON SampleSingleCell
FOR EACH ROW
  BEGIN
	DECLARE log_message varchar(500) CHARACTER SET utf8;
    SET log_message = CONCAT_WS(', ',
      CASE WHEN (NEW.initialCellConcentration IS NULL) <> (OLD.initialCellConcentration IS NULL) OR NEW.initialCellConcentration <> OLD.initialCellConcentration THEN CONCAT('initial cell concentration: ', COALESCE(decimalToString(OLD.initialCellConcentration), 'n/a'), ' → ', COALESCE(decimalToString(NEW.initialCellConcentration), 'n/a')) END,
      CASE WHEN NEW.digestion <> OLD.digestion THEN CONCAT('digestion: ', OLD.digestion, ' → ', NEW.digestion) END);
    IF log_message IS NOT NULL AND log_message <> '' THEN
      INSERT INTO SampleChangeLog(sampleId, columnsChanged, userId, message, changeTime)
      SELECT
        NEW.sampleId,
        COALESCE(CONCAT_WS(',',
          CASE WHEN (NEW.initialCellConcentration IS NULL) <> (OLD.initialCellConcentration IS NULL) OR NEW.initialCellConcentration <> OLD.initialCellConcentration THEN 'initialConcentration' END,
          CASE WHEN NEW.digestion <> OLD.digestion THEN 'digestion' END
        ), ''),
        lastModifier,
        log_message,
        lastModified
      FROM Sample WHERE sampleId = NEW.sampleId;
    END IF;
  END//

DROP TRIGGER IF EXISTS SampleStockSingleCellChange//
CREATE TRIGGER SampleStockSingleCellChange BEFORE UPDATE ON SampleStockSingleCell
FOR EACH ROW
  BEGIN
	DECLARE log_message varchar(500) CHARACTER SET utf8;
    SET log_message = CONCAT_WS(', ',
      CASE WHEN (NEW.targetCellRecovery IS NULL) <> (OLD.targetCellRecovery IS NULL) OR NEW.targetCellRecovery <> OLD.targetCellRecovery THEN CONCAT('target cell recovery: ', COALESCE(decimalToString(OLD.targetCellRecovery), 'n/a'), ' → ', COALESCE(decimalToString(NEW.targetCellRecovery), 'n/a')) END,
      CASE WHEN (NEW.cellViability IS NULL) <> (OLD.cellViability IS NULL) OR NEW.cellViability <> OLD.cellViability THEN CONCAT('cell viability: ', COALESCE(decimalToString(OLD.cellViability), 'n/a'), ' → ', COALESCE(decimalToString(NEW.cellViability), 'n/a')) END,
      CASE WHEN (NEW.loadingCellConcentration IS NULL) <> (OLD.loadingCellConcentration IS NULL) OR NEW.loadingCellConcentration <> OLD.loadingCellConcentration THEN CONCAT('loading cell concentration: ', COALESCE(decimalToString(OLD.loadingCellConcentration), 'n/a'), ' → ', COALESCE(decimalToString(NEW.loadingCellConcentration), 'n/a')) END);
    IF log_message IS NOT NULL AND log_message <> '' THEN
      INSERT INTO SampleChangeLog(sampleId, columnsChanged, userId, message, changeTime)
      SELECT
        NEW.sampleId,
        COALESCE(CONCAT_WS(',',
          CASE WHEN (NEW.targetCellRecovery IS NULL) <> (OLD.targetCellRecovery IS NULL) OR NEW.targetCellRecovery <> OLD.targetCellRecovery THEN 'targetCellRecovery' END,
          CASE WHEN (NEW.cellViability IS NULL) <> (OLD.cellViability IS NULL) OR NEW.cellViability <> OLD.cellViability THEN 'cellViability' END,
          CASE WHEN (NEW.loadingCellConcentration IS NULL) <> (OLD.loadingCellConcentration IS NULL) OR NEW.loadingCellConcentration <> OLD.loadingCellConcentration THEN 'loadingCellConcentration' END
        ), ''),
        lastModifier,
        log_message,
        lastModified
      FROM Sample WHERE sampleId = NEW.sampleId;
    END IF;
  END//

DROP TRIGGER IF EXISTS SampleAliquotSingleCellChange//
CREATE TRIGGER SampleAliquotSingleCellChange BEFORE UPDATE ON SampleAliquotSingleCell
FOR EACH ROW
  BEGIN
    DECLARE log_message varchar(500) CHARACTER SET utf8;
    SET log_message = CONCAT_WS(', ',
      CASE WHEN (NEW.inputIntoLibrary IS NULL) <> (OLD.inputIntoLibrary IS NULL) OR NEW.inputIntoLibrary <> OLD.inputIntoLibrary THEN CONCAT('input into library: ', COALESCE(decimalToString(OLD.inputIntoLibrary), 'n/a'), ' → ', COALESCE(decimalToString(NEW.inputIntoLibrary), 'n/a')) END);
    IF log_message IS NOT NULL AND log_message <> '' THEN
      INSERT INTO SampleChangeLog(sampleId, columnsChanged, userId, message, changeTime)
      SELECT
        NEW.sampleId,
        COALESCE(CONCAT_WS(',',
          CASE WHEN (NEW.inputIntoLibrary IS NULL) <> (OLD.inputIntoLibrary IS NULL) OR NEW.inputIntoLibrary <> OLD.inputIntoLibrary THEN 'inputINtoLibrary' END
        ), ''),
        lastModifier,
        log_message,
        lastModified
      FROM Sample WHERE sampleId = NEW.sampleId;
    END IF;
  END//

DROP TRIGGER IF EXISTS SampleInsert//
CREATE TRIGGER SampleInsert AFTER INSERT ON Sample
FOR EACH ROW
  INSERT INTO SampleChangeLog(sampleId, columnsChanged, userId, message, changeTime) VALUES (
    NEW.sampleId,
    '',
    NEW.lastModifier,
    'Sample created.',
    NEW.lastModified)//

DROP TRIGGER IF EXISTS DetailedSampleInsert//
CREATE TRIGGER DetailedSampleInsert AFTER INSERT ON DetailedSample
FOR EACH ROW
  CALL updateSampleHierarchy(NEW.sampleId)//

-- Need to fix for tissues because DetailedSample record is created before SampleTissue
DROP TRIGGER IF EXISTS SampleTissueInsert//
CREATE TRIGGER SampleTissueInsert AFTER INSERT ON SampleTissue
FOR EACH ROW
  UPDATE SampleHierarchy SET tissueId = NEW.sampleId WHERE sampleId = NEW.sampleId//

DELIMITER ;
-- EndNoTest
